#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © 2015 uralbash <root@uralbash.ru>
#
# Distributed under terms of the MIT license.
import subprocess

import docutils
from docutils.parsers.rst.roles import set_classes

from .collection import LINKS

SETTING_LINKS = 'links_collection'


class CaseInsensitiveDict(dict):

    """Basic case insensitive dict with strings only keys."""

    proxy = {}

    def __init__(self, data):
        self.proxy = dict((k.lower(), k) for k in data)
        for k in data:
            self[k] = data[k]

    def __contains__(self, k):
        return k.lower() in self.proxy

    def __delitem__(self, k):
        key = self.proxy[k.lower()]
        super(CaseInsensitiveDict, self).__delitem__(key)
        del self.proxy[k.lower()]

    def __getitem__(self, k):
        key = self.proxy[k.lower()]
        return super(CaseInsensitiveDict, self).__getitem__(key)

    def get(self, k, default=None):
        return self[k] if k in self else default

    def __setitem__(self, k, v):
        super(CaseInsensitiveDict, self).__setitem__(k, v)
        self.proxy[k.lower()] = k


def external_links(links={}):
    def role(
        role, rawtext, text,
        lineno, inliner, options={}, content=[]
    ):
        """
        Example:

            See code there :l:`Nginx`.
        """
        merged_links = dict(list(LINKS.items()) + list(links.items()))
        link = CaseInsensitiveDict(merged_links)[text.lower()]
        set_classes(options)
        node = docutils.nodes.reference(
            rawtext,
            docutils.utils.unescape(text),
            refuri=link,
            **options
        )
        return [node], []
    return role


def pypi(
    role, rawtext, text,
    lineno, inliner, options={}, content=[]
):
    """
    Example:

        See code there :pypi:`sqlalchemy_mptt`.
    """
    set_classes(options)
    link = u"https://pypi.python.org/pypi/{}".format(text)
    node = docutils.nodes.reference(
        rawtext,
        docutils.utils.unescape(text),
        refuri=link,
        **options
    )
    return [node], []


def man(
    role, rawtext, text,
    lineno, inliner, options={}, content=[]
):
    """
    Example:

    See code there :man:`open`.
    """
    set_classes(options)
    link = "https://www.freebsd.org/cgi/man.cgi?query={}".format(text)
    node = docutils.nodes.reference(
        rawtext,
        docutils.utils.unescape(text),
        refuri=link,
        **options
    )
    return [node], []


def github(
    role, rawtext, text,
    lineno, inliner, options={}, content=[]
):
    """
    Example:

    See code there :github:`ITCase/sacrud`.
    """
    set_classes(options)
    link = "https://github.com/{}".format(text)
    node = docutils.nodes.reference(
        rawtext,
        docutils.utils.unescape(text),
        refuri=link,
        **options
    )
    return [node], []


def add_role(app, env, docname):
    links = getattr(env.app.config, SETTING_LINKS, {})
    app.add_role('l', external_links(links))


def setup(app):
    app.add_config_value(SETTING_LINKS, {}, False)
    app.connect('env-before-read-docs', add_role)
    app.add_role('pypi', pypi)
    app.add_role('man', man)
    app.add_role('github', github)



LINKS = {
    # Example
    'YotaPhone': 'http://yotaphone.com/',

    # Languages
    'Python': 'https://www.python.org/',
    'Java': 'http://www.java.com/',
    'Perl': 'http://perl.org/',
    'PHP': 'http://php.net/',
    'go': 'https://golang.org/',
    'golang': 'https://golang.org/',

    # OS
    'Linux': 'https://www.kernel.org/',
    'Debian': 'https://www.debian.org/',
    'Ubuntu': 'http://www.ubuntu.com/',
    'MacOS': 'http://www.apple.com/osx/',
    'Windows': 'http://windows.microsoft.com/',

    # DE
    'Qtile': 'http://qtile.org',

    # Databases
    'Postgres': 'http://www.postgresql.org/',
    'PostgreSQL': 'http://www.postgresql.org/',
    'sqlite': 'http://www.sqlite.org/',

    # NoSQL
    'CouchDB': 'http://couchdb.apache.org/',

    # Vendor
    'Apple': 'http://www.apple.com/',
    'Microsoft': 'http://microsoft.com/',
    'Digia': 'http://www.digia.com/',

    # WEB Frameworks
    'Django': 'https://www.djangoproject.com/',
    'Pylons': 'http://www.pylonsproject.org/',
    'Pyramid': 'http://www.pylonsproject.org/',
    'Flask': 'http://flask.pocoo.org/',
    'aiohttp': 'https://github.com/KeepSafe/aiohttp/',

    # Web server
    'nginx': 'http://nginx.org',
    'Apache': 'http://www.apache.org',

    # DE Frameworks
    'Qt': 'http://www.qt.io',
    'Qt4': 'http://www.qt.io',
    'Qt5': 'http://www.qt.io',

    # ORM
    'SQLAlchemy': 'http://www.sqlalchemy.org',

    # Template engine
    'Mako': 'http://docs.makotemplates.org/en/latest/',
    'Jinja': 'http://jinja.pocoo.org/',
    'Jinja2': 'http://jinja.pocoo.org/',

    # W3C
    'W3C': 'http://www.w3.org/',
    'CSS': 'http://www.w3.org/Style/CSS/',

    # JavaScript
    'JQuery': 'https://jquery.org/',

    # CVS
    'github': 'https://github.com/',

    # GIS
    'OpenStreetMap': 'http://www.openstreetmap.org',
    'OSM': 'http://www.openstreetmap.org',

    # packages
    'PyPi': 'https://pypi.python.org/',

    # editor
    'vi': 'http://www.vim.org',
    'vim': 'http://www.vim.org',

    # Virtualization
    'docker': 'http://docker.io/',
}



import os

from setuptools import find_packages, setup

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()


setup(
    name='sphinx-links',
    version='0.0.5',
    description='Added external links to Sphinx project as :l:`name of link`.',
    long_description=README,
    classifiers=[
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "Programming Language :: Python",
        "License :: Repoze Public License",
        "Topic :: Internet",
        "Topic :: Software Development :: Documentation",
    ],
    keywords='sphinx documentation',
    author="uralbash",
    author_email="sacrud@uralbash.ru",
    url="https://github.com/ITCase/sphinx-links",
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
)
